@@ -1,27 +1,31 @@
 /* Debugging extensions for Guile
- * Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002, 2003, 2006, 2008, 2009, 2010, 2011, 2012, 2013 Free Software Foundation
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+
+   Copyright 1995-2003,2006,2008-2013,2018
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
+#include <errno.h>
+
 #ifdef HAVE_GETRLIMIT
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -32,38 +36,38 @@
 # include <windows.h>
 #endif
 
-#include "libguile/_scm.h"
-#include "libguile/async.h"
-#include "libguile/eval.h"
-#include "libguile/list.h"
-#include "libguile/stackchk.h"
-#include "libguile/throw.h"
-#include "libguile/macros.h"
-#include "libguile/smob.h"
-#include "libguile/struct.h"
-#include "libguile/procprop.h"
-#include "libguile/srcprop.h"
-#include "libguile/alist.h"
-#include "libguile/continuations.h"
-#include "libguile/strports.h"
-#include "libguile/read.h"
-#include "libguile/feature.h"
-#include "libguile/dynwind.h"
-#include "libguile/modules.h"
-#include "libguile/ports.h"
-#include "libguile/root.h"
-#include "libguile/fluids.h"
-#include "libguile/programs.h"
-#include "libguile/memoize.h"
-#include "libguile/vm.h"
+#include "alist.h"
+#include "async.h"
+#include "continuations.h"
+#include "dynwind.h"
+#include "eval.h"
+#include "feature.h"
+#include "fluids.h"
+#include "gsubr.h"
+#include "list.h"
+#include "macros.h"
+#include "memoize.h"
+#include "modules.h"
+#include "pairs.h"
+#include "ports.h"
+#include "private-options.h"
+#include "procprop.h"
+#include "programs.h"
+#include "read.h"
+#include "smob.h"
+#include "srcprop.h"
+#include "stackchk.h"
+#include "strports.h"
+#include "struct.h"
+#include "throw.h"
+#include "variable.h"
+#include "vm.h"
+
+#include "debug.h"
 
-#include "libguile/validate.h"
-#include "libguile/debug.h"
 
-#include "libguile/private-options.h"
 
 
-
 /*
  * Debugging options.
  */
@@ -109,13 +113,9 @@ SCM_DEFINE (scm_debug_options, "debug-options-interface", 0, 1, 0,
 {
   SCM ans;
 
-  scm_dynwind_begin (0);
-  scm_dynwind_critical_section (SCM_BOOL_F);
-
   ans = scm_options (setting, scm_debug_opts, FUNC_NAME);
   scm_stack_checking_enabled_p = SCM_STACK_CHECKING_P;
 
-  scm_dynwind_end ();
   return ans;
 }
 #undef FUNC_NAME
@@ -222,11 +222,5 @@ scm_init_debug ()
 
   scm_add_feature ("debug-extensions");
 
-#include "libguile/debug.x"
+#include "debug.x"
 }
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
