@@ -1,22 +1,22 @@
 /* "net_db.c" network database support
- * Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2006, 2009,
- *   2010, 2011, 2012, 2013 Free Software Foundation, Inc.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+   Copyright 1995-2001,2006,2009-2013,2018
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 
@@ -33,11 +33,7 @@
 
 #include <verify.h>
 #include <errno.h>
-
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif
-
 #include <sys/types.h>
 
 #include <sys/socket.h>
@@ -45,15 +41,21 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#include "libguile/_scm.h"
-#include "libguile/feature.h"
-#include "libguile/strings.h"
-#include "libguile/vectors.h"
-#include "libguile/dynwind.h"
+#include "boolean.h"
+#include "dynwind.h"
+#include "feature.h"
+#include "gsubr.h"
+#include "list.h"
+#include "modules.h"
+#include "numbers.h"
+#include "pairs.h"
+#include "socket.h"
+#include "strings.h"
+#include "symbols.h"
+#include "throw.h"
+#include "vectors.h"
 
-#include "libguile/validate.h"
-#include "libguile/net_db.h"
-#include "libguile/socket.h"
+#include "net_db.h"
 
 
 #if defined (HAVE_H_ERRNO)
@@ -722,11 +724,5 @@ void
 scm_init_net_db ()
 {
   scm_add_feature ("net-db");
-#include "libguile/net_db.x"
+#include "net_db.x"
 }
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
