@@ -1,6 +1,6 @@
 ;;; Continuation-passing style (CPS) intermediate language (IL)
 
-;; Copyright (C) 2013, 2014, 2015, 2017, 2018 Free Software Foundation, Inc.
+;; Copyright (C) 2013, 2014, 2015, 2017, 2018, 2019, 2020, 2021 Free Software Foundation, Inc.
 
 ;;;; This library is free software; you can redistribute it and/or
 ;;;; modify it under the terms of the GNU Lesser General Public
@@ -29,35 +29,39 @@
   #:use-module (language cps)
   #:use-module (language cps intset)
   #:use-module (language cps intmap)
+  #:use-module (language cps graphs)
   #:export (;; Fresh names.
             label-counter var-counter
             fresh-label fresh-var
             with-fresh-name-state compute-max-label-and-var
             let-fresh
 
-            ;; Various utilities.
-            fold1 fold2
-            trivial-intset
-            intmap-map
-            intmap-keys
-            invert-bijection invert-partition
-            intset->intmap
-            worklist-fold
-            fixpoint
-
-            ;; Flow analysis.
+            ;; Graphs.
             compute-function-body
+            compute-singly-referenced-labels
             compute-reachable-functions
             compute-successors
-            invert-graph
             compute-predecessors
-            compute-reverse-post-order
-            compute-strongly-connected-components
-            compute-sorted-strongly-connected-components
             compute-idoms
             compute-dom-edges
-            solve-flow-equations
-            ))
+            compute-defs-and-uses
+            compute-var-representations)
+  #:re-export (fold1 fold2
+               trivial-intset
+               intmap-map
+               intmap-keys
+               invert-bijection invert-partition
+               intset->intmap
+               intmap-select
+               worklist-fold
+               fixpoint
+
+               ;; Flow analysis.
+               invert-graph
+               compute-reverse-post-order
+               compute-strongly-connected-components
+               compute-sorted-strongly-connected-components
+               solve-flow-equations))
 
 (define label-counter (make-parameter #f))
 (define var-counter (make-parameter #f))
@@ -98,87 +102,6 @@
                        conts
                        -1)))
 
-(define-inlinable (fold1 f l s0)
-  (let lp ((l l) (s0 s0))
-    (match l
-      (() s0)
-      ((elt . l) (lp l (f elt s0))))))
-
-(define-inlinable (fold2 f l s0 s1)
-  (let lp ((l l) (s0 s0) (s1 s1))
-    (match l
-      (() (values s0 s1))
-      ((elt . l)
-       (call-with-values (lambda () (f elt s0 s1))
-         (lambda (s0 s1)
-           (lp l s0 s1)))))))
-
-(define (trivial-intset set)
-  "Returns the sole member of @var{set}, if @var{set} has exactly one
-member, or @code{#f} otherwise."
-  (let ((first (intset-next set)))
-    (and first
-         (not (intset-next set (1+ first)))
-         first)))
-
-(define (intmap-map proc map)
-  (persistent-intmap
-   (intmap-fold (lambda (k v out) (intmap-add! out k (proc k v)))
-                map
-                empty-intmap)))
-
-(define (intmap-keys map)
-  "Return an intset of the keys in @var{map}."
-  (persistent-intset
-   (intmap-fold (lambda (k v keys) (intset-add! keys k)) map empty-intset)))
-
-(define (invert-bijection map)
-  "Assuming the values of @var{map} are integers and are unique, compute
-a map in which each value maps to its key.  If the values are not
-unique, an error will be signalled."
-  (intmap-fold (lambda (k v out) (intmap-add out v k)) map empty-intmap))
-
-(define (invert-partition map)
-  "Assuming the values of @var{map} are disjoint intsets, compute a map
-in which each member of each set maps to its key.  If the values are not
-disjoint, an error will be signalled."
-  (intmap-fold (lambda (k v* out)
-                 (intset-fold (lambda (v out) (intmap-add out v k)) v* out))
-               map empty-intmap))
-
-(define (intset->intmap f set)
-  (persistent-intmap
-   (intset-fold (lambda (label preds)
-                  (intmap-add! preds label (f label)))
-                set empty-intmap)))
-
-(define worklist-fold
-  (case-lambda
-    ((f in out)
-     (let lp ((in in) (out out))
-       (if (eq? in empty-intset)
-           out
-           (call-with-values (lambda () (f in out)) lp))))
-    ((f in out0 out1)
-     (let lp ((in in) (out0 out0) (out1 out1))
-       (if (eq? in empty-intset)
-           (values out0 out1)
-           (call-with-values (lambda () (f in out0 out1)) lp))))))
-
-(define fixpoint
-  (case-lambda
-    ((f x)
-     (let lp ((x x))
-       (let ((x* (f x)))
-         (if (eq? x x*) x* (lp x*)))))
-    ((f x0 x1)
-     (let lp ((x0 x0) (x1 x1))
-       (call-with-values (lambda () (f x0 x1))
-         (lambda (x0* x1*)
-           (if (and (eq? x0 x0*) (eq? x1 x1*))
-               (values x0* x1*)
-               (lp x0* x1*))))))))
-
 (define (compute-function-body conts kfun)
   (persistent-intset
    (let visit-cont ((label kfun) (labels empty-intset))
@@ -204,11 +127,44 @@ disjoint, an error will be signalled."
                (visit-cont k labels))
               (($ $branch kf kt)
                (visit-cont kf (visit-cont kt labels)))
+              (($ $switch kf kt*)
+               (visit-cont kf (fold1 visit-cont kt* labels)))
               (($ $prompt k kh)
                (visit-cont k (visit-cont kh labels)))
               (($ $throw)
                labels))))))))))
 
+(define (compute-singly-referenced-labels conts)
+  "Compute the set of labels in CONTS that have exactly one
+predecessor."
+  (define (add-ref label cont single multiple)
+    (define (ref k single multiple)
+      (if (intset-ref single k)
+          (values single (intset-add! multiple k))
+          (values (intset-add! single k) multiple)))
+    (define (ref0) (values single multiple))
+    (define (ref1 k) (ref k single multiple))
+    (define (ref2 k k*)
+      (if k*
+          (let-values (((single multiple) (ref k single multiple)))
+            (ref k* single multiple))
+          (ref1 k)))
+    (match cont
+      (($ $kreceive arity k) (ref1 k))
+      (($ $kfun src meta self ktail kclause) (ref2 ktail kclause))
+      (($ $ktail) (ref0))
+      (($ $kclause arity kbody kalt) (ref2 kbody kalt))
+      (($ $kargs names syms ($ $continue k)) (ref1 k))
+      (($ $kargs names syms ($ $branch kf kt)) (ref2 kf kt))
+      (($ $kargs names syms ($ $switch kf kt*))
+       (fold2 ref (cons kf kt*) single multiple))
+      (($ $kargs names syms ($ $prompt k kh)) (ref2 k kh))
+      (($ $kargs names syms ($ $throw)) (ref0))))
+  (let*-values (((single multiple) (values empty-intset empty-intset))
+                ((single multiple) (intmap-fold add-ref conts single multiple)))
+    (intset-subtract (persistent-intset single)
+                     (persistent-intset multiple))))
+
 (define* (compute-reachable-functions conts #:optional (kfun 0))
   "Compute a mapping LABEL->LABEL..., where each key is a reachable
 $kfun and each associated value is the body of the function, as an
@@ -258,6 +214,10 @@ intset."
       (define (propagate2 succ0 succ1)
         (let ((succs (intmap-add! succs label (intset succ0 succ1))))
           (visit succ1 (visit succ0 succs))))
+      (define (propagate* k*)
+        (define (list->intset ls)
+          (fold1 (lambda (elt set) (intset-add set elt)) ls empty-intset))
+        (fold1 visit k* (intmap-add! succs label (list->intset k*))))
       (if (intmap-ref succs label (lambda (_) #f))
           succs
           (match (intmap-ref conts label)
@@ -265,6 +225,7 @@ intset."
              (match term
                (($ $continue k) (propagate1 k))
                (($ $branch kf kt) (propagate2 kf kt))
+               (($ $switch kf kt*) (propagate* (cons kf kt*)))
                (($ $prompt k kh) (propagate2 k kh))
                (($ $throw) (propagate0))))
             (($ $kreceive arity k)
@@ -300,111 +261,13 @@ intset."
        (match term
          (($ $continue k)   (add-pred k preds))
          (($ $branch kf kt) (add-pred kf (add-pred kt preds)))
+         (($ $switch kf kt*) (fold1 add-pred (cons kf kt*) preds))
          (($ $prompt k kh)  (add-pred k (add-pred kh preds)))
          (($ $throw)        preds)))))
   (persistent-intmap
    (intset-fold add-preds labels
                 (intset->intmap (lambda (label) '()) labels))))
 
-(define (compute-reverse-post-order succs start)
-  "Compute a reverse post-order numbering for a depth-first walk over
-nodes reachable from the start node."
-  (let visit ((label start) (order '()) (visited empty-intset))
-    (call-with-values
-        (lambda ()
-          (intset-fold (lambda (succ order visited)
-                         (if (intset-ref visited succ)
-                             (values order visited)
-                             (visit succ order visited)))
-                       (intmap-ref succs label)
-                       order
-                       (intset-add! visited label)))
-      (lambda (order visited)
-        ;; After visiting successors, add label to the reverse post-order.
-        (values (cons label order) visited)))))
-
-(define (invert-graph succs)
-  "Given a graph PRED->SUCC..., where PRED is a label and SUCC... is an
-intset of successors, return a graph SUCC->PRED...."
-  (intmap-fold (lambda (pred succs preds)
-                 (intset-fold
-                  (lambda (succ preds)
-                    (intmap-add preds succ pred intset-add))
-                  succs
-                  preds))
-               succs
-               (intmap-map (lambda (label _) empty-intset) succs)))
-
-(define (compute-strongly-connected-components succs start)
-  "Given a LABEL->SUCCESSOR... graph, compute a SCC->LABEL... map
-partitioning the labels into strongly connected components (SCCs)."
-  (let ((preds (invert-graph succs)))
-    (define (visit-scc scc sccs-by-label)
-      (let visit ((label scc) (sccs-by-label sccs-by-label))
-        (if (intmap-ref sccs-by-label label (lambda (_) #f))
-            sccs-by-label
-            (intset-fold visit
-                         (intmap-ref preds label)
-                         (intmap-add sccs-by-label label scc)))))
-    (intmap-fold
-     (lambda (label scc sccs)
-       (let ((labels (intset-add empty-intset label)))
-         (intmap-add sccs scc labels intset-union)))
-     (fold visit-scc empty-intmap (compute-reverse-post-order succs start))
-     empty-intmap)))
-
-(define (compute-sorted-strongly-connected-components edges)
-  "Given a LABEL->SUCCESSOR... graph, return a list of strongly
-connected components in sorted order."
-  (define nodes
-    (intmap-keys edges))
-  ;; Add a "start" node that links to all nodes in the graph, and then
-  ;; remove it from the result.
-  (define start
-    (if (eq? nodes empty-intset)
-        0
-        (1+ (intset-prev nodes))))
-  (define components
-    (intmap-remove
-     (compute-strongly-connected-components (intmap-add edges start nodes)
-                                            start)
-     start))
-  (define node-components
-    (intmap-fold (lambda (id nodes out)
-                   (intset-fold (lambda (node out) (intmap-add out node id))
-                                nodes out))
-                 components
-                 empty-intmap))
-  (define (node-component node)
-    (intmap-ref node-components node))
-  (define (component-successors id nodes)
-    (intset-remove
-     (intset-fold (lambda (node out)
-                    (intset-fold
-                     (lambda (successor out)
-                       (intset-add out (node-component successor)))
-                     (intmap-ref edges node)
-                     out))
-                  nodes
-                  empty-intset)
-     id))
-  (define component-edges
-    (intmap-map component-successors components))
-  (define preds
-    (invert-graph component-edges))
-  (define roots
-    (intmap-fold (lambda (id succs out)
-                   (if (eq? empty-intset succs)
-                       (intset-add out id)
-                       out))
-                 component-edges
-                 empty-intset))
-  ;; As above, add a "start" node that links to the roots, and remove it
-  ;; from the result.
-  (match (compute-reverse-post-order (intmap-add preds start roots) start)
-    (((? (lambda (id) (eqv? id start))) . ids)
-     (map (lambda (id) (intmap-ref components id)) ids))))
-
 ;; Precondition: For each function in CONTS, the continuation names are
 ;; topologically sorted.
 (define (compute-idoms conts kfun)
@@ -454,45 +317,141 @@ connected components in sorted order."
                 idoms
                 empty-intmap)))
 
-(define (intset-pop set)
-  (match (intset-next set)
-    (#f (values set #f))
-    (i (values (intset-remove set i) i))))
+(define (compute-defs-and-uses cps)
+  "Return two LABEL->VAR... maps indicating values defined at and used
+by a label, respectively."
+  (define (vars->intset vars)
+    (fold (lambda (var set) (intset-add set var)) empty-intset vars))
+  (define-syntax-rule (persistent-intmap2 exp)
+    (call-with-values (lambda () exp)
+      (lambda (a b)
+        (values (persistent-intmap a) (persistent-intmap b)))))
+  (persistent-intmap2
+   (intmap-fold
+    (lambda (label cont defs uses)
+      (define (get-defs k)
+        (match (intmap-ref cps k)
+          (($ $kargs names vars) (vars->intset vars))
+          (_ empty-intset)))
+      (define (return d u)
+        (values (intmap-add! defs label d)
+                (intmap-add! uses label u)))
+      (match cont
+        (($ $kfun src meta self tail clause)
+         (return (intset-union
+                  (if clause (get-defs clause) empty-intset)
+                  (if self (intset self) empty-intset))
+                 empty-intset))
+        (($ $kargs _ _ ($ $continue k src exp))
+         (match exp
+           ((or ($ $const) ($ $const-fun) ($ $code))
+            (return (get-defs k) empty-intset))
+           (($ $call proc args)
+            (return (get-defs k) (intset-add (vars->intset args) proc)))
+           (($ $callk _ proc args)
+            (let ((args (vars->intset args)))
+              (return (get-defs k) (if proc (intset-add args proc) args))))
+           (($ $primcall name param args)
+            (return (get-defs k) (vars->intset args)))
+           (($ $values args)
+            (return (get-defs k) (vars->intset args)))))
+        (($ $kargs _ _ ($ $branch kf kt src op param args))
+         (return empty-intset (vars->intset args)))
+        (($ $kargs _ _ ($ $switch kf kt* src arg))
+         (return empty-intset (intset arg)))
+        (($ $kargs _ _ ($ $prompt k kh src escape? tag))
+         (return empty-intset (intset tag)))
+        (($ $kargs _ _ ($ $throw src op param args))
+         (return empty-intset (vars->intset args)))
+        (($ $kclause arity body alt)
+         (return (get-defs body) empty-intset))
+        (($ $kreceive arity kargs)
+         (return (get-defs kargs) empty-intset))
+        (($ $ktail)
+         (return empty-intset empty-intset))))
+    cps
+    empty-intmap
+    empty-intmap)))
 
-(define* (solve-flow-equations succs in out kill gen subtract add meet
-                               #:optional (worklist (intmap-keys succs)))
-  "Find a fixed point for flow equations for SUCCS, where INIT is the
-initial state at each node in SUCCS.  KILL and GEN are intmaps
-indicating the state that is killed or defined at every node, and
-SUBTRACT, ADD, and MEET operates on that state."
-  (define (visit label in out)
-    (let* ((in-1 (intmap-ref in label))
-           (kill-1 (intmap-ref kill label))
-           (gen-1 (intmap-ref gen label))
-           (out-1 (intmap-ref out label))
-           (out-1* (add (subtract in-1 kill-1) gen-1)))
-      (if (eq? out-1 out-1*)
-          (values empty-intset in out)
-          (let ((out (intmap-replace! out label out-1*)))
-            (call-with-values
-                (lambda ()
-                  (intset-fold (lambda (succ in changed)
-                                 (let* ((in-1 (intmap-ref in succ))
-                                        (in-1* (meet in-1 out-1*)))
-                                   (if (eq? in-1 in-1*)
-                                       (values in changed)
-                                       (values (intmap-replace! in succ in-1*)
-                                               (intset-add changed succ)))))
-                               (intmap-ref succs label) in empty-intset))
-              (lambda (in changed)
-                (values changed in out)))))))
-
-  (let run ((worklist worklist) (in in) (out out))
-    (call-with-values (lambda () (intset-pop worklist))
-      (lambda (worklist popped)
-        (if popped
-            (call-with-values (lambda () (visit popped in out))
-              (lambda (changed in out)
-                (run (intset-union worklist changed) in out)))
-            (values (persistent-intmap in)
-                    (persistent-intmap out)))))))
+(define (compute-var-representations cps)
+  (define (get-defs k)
+    (match (intmap-ref cps k)
+      (($ $kargs names vars) vars)
+      (_ '())))
+  (intmap-fold
+   (lambda (label cont representations)
+     (match cont
+       (($ $kargs _ _ ($ $continue k _ exp))
+        (match (get-defs k)
+          (() representations)
+          ((var)
+           (match exp
+             (($ $values (arg))
+              (intmap-add representations var
+                          (intmap-ref representations arg)))
+             (($ $callk)
+              (intmap-add representations var 'scm))
+             (($ $primcall (or 'scm->f64 'load-f64 's64->f64
+                               'f32-ref 'f64-ref
+                               'fadd 'fsub 'fmul 'fdiv 'fsqrt 'fabs
+                               'ffloor 'fceiling
+                               'fsin 'fcos 'ftan 'fasin 'facos 'fatan 'fatan2))
+              (intmap-add representations var 'f64))
+             (($ $primcall (or 'scm->u64 'scm->u64/truncate 'load-u64
+                               's64->u64
+                               'assume-u64
+                               'uadd 'usub 'umul
+                               'ulogand 'ulogior 'ulogxor 'ulogsub 'ursh 'ulsh
+                               'uadd/immediate 'usub/immediate 'umul/immediate
+                               'ursh/immediate 'ulsh/immediate
+                               'u8-ref 'u16-ref 'u32-ref 'u64-ref
+                               'word-ref 'word-ref/immediate
+                               'untag-char))
+              (intmap-add representations var 'u64))
+             (($ $primcall (or 'untag-fixnum
+                               'assume-s64
+                               'scm->s64 'load-s64 'u64->s64
+                               'srsh 'srsh/immediate
+                               's8-ref 's16-ref 's32-ref 's64-ref))
+              (intmap-add representations var 's64))
+             (($ $primcall (or 'pointer-ref/immediate
+                               'tail-pointer-ref/immediate))
+              (intmap-add representations var 'ptr))
+             (($ $code)
+              (intmap-add representations var 'u64))
+             (_
+              (intmap-add representations var 'scm))))
+          (vars
+           (match exp
+             (($ $values args)
+              (fold (lambda (arg var representations)
+                      (intmap-add representations var
+                                  (intmap-ref representations arg)))
+                    representations args vars))
+             (($ $callk)
+              (fold1 (lambda (var representations)
+                      (intmap-add representations var 'scm))
+                     vars representations))))))
+       (($ $kargs _ _ (or ($ $branch) ($ $switch) ($ $prompt) ($ $throw)))
+        representations)
+       (($ $kfun src meta self tail entry)
+        (let* ((representations (if self
+                                   (intmap-add representations self 'scm)
+                                   representations))
+               (defs (get-defs entry))
+               (reprs (or (assq-ref meta 'arg-representations)
+                          (map (lambda (_) 'scm) defs))))
+          (fold (lambda (var repr representations)
+                   (intmap-add representations var repr))
+                representations defs reprs)))
+       (($ $kclause arity body alt)
+        (fold1 (lambda (var representations)
+                 (intmap-add representations var 'scm))
+               (get-defs body) representations))
+       (($ $kreceive arity kargs)
+        (fold1 (lambda (var representations)
+                 (intmap-add representations var 'scm))
+               (get-defs kargs) representations))
+       (($ $ktail) representations)))
+   cps
+   empty-intmap))
