@@ -1,6 +1,6 @@
 ;;;; ports.scm --- R6RS port API                    -*- coding: utf-8 -*-
 
-;;;;	Copyright (C) 2009, 2010, 2011 Free Software Foundation, Inc.
+;;;; Copyright (C) 2009-2011, 2013, 2019 Free Software Foundation, Inc.
 ;;;;
 ;;;; This library is free software; you can redistribute it and/or
 ;;;; modify it under the terms of the GNU Lesser General Public
@@ -36,6 +36,9 @@
           transcoder-error-handling-mode native-transcoder
           latin-1-codec utf-8-codec utf-16-codec
            
+          ;; transcoding bytevectors
+          bytevector->string string->bytevector
+
           ;; input & output ports
           port? input-port? output-port?
           port-eof?
@@ -63,8 +66,13 @@
           call-with-bytevector-output-port
           call-with-string-output-port
           make-custom-textual-output-port
+          output-port-buffer-mode
           flush-output-port
-           
+
+          ;; input/output ports
+          open-file-input/output-port
+          make-custom-binary-input/output-port
+
           ;; binary output
           put-u8 put-bytevector
 
@@ -97,12 +105,16 @@
           make-i/o-file-does-not-exist-error
           &i/o-port i/o-port-error? make-i/o-port-error
           i/o-error-port
-          &i/o-decoding-error i/o-decoding-error?
+          &i/o-decoding i/o-decoding-error?
           make-i/o-decoding-error
-          &i/o-encoding-error i/o-encoding-error?
+          &i/o-encoding i/o-encoding-error?
           make-i/o-encoding-error i/o-encoding-error-char)
   (import (ice-9 binary-ports)
           (only (rnrs base) assertion-violation)
+          (only (ice-9 ports internal)
+                port-write-buffer port-buffer-bytevector port-line-buffered?)
+          (only (rnrs bytevectors) bytevector-length)
+          (prefix (ice-9 iconv) iconv:)
           (rnrs enums)
           (rnrs records syntactic)
           (rnrs exceptions)
@@ -164,6 +176,33 @@
 (define (utf-16-codec)
   "UTF-16")
 
+
+;;;
+;;; Transcoding bytevectors
+;;;
+
+(define (string->bytevector str transcoder)
+  "Encode @var{str} using @var{transcoder}, returning a bytevector."
+  (iconv:string->bytevector
+   str
+   (transcoder-codec transcoder)
+   (case (transcoder-error-handling-mode transcoder)
+     ((raise) 'error)
+     ((replace) 'substitute)
+     (else (error "unsupported error handling mode"
+                  (transcoder-error-handling-mode transcoder))))))
+
+(define (bytevector->string bv transcoder)
+  "Decode @var{bv} using @var{transcoder}, returning a string."
+  (iconv:bytevector->string
+   bv
+   (transcoder-codec transcoder)
+   (case (transcoder-error-handling-mode transcoder)
+     ((raise) 'error)
+     ((replace) 'substitute)
+     (else (error "unsupported error handling mode"
+                  (transcoder-error-handling-mode transcoder))))))
+
 
 ;;;
 ;;; Internal helpers
@@ -217,28 +256,34 @@
 (define (port-transcoder port)
   "Return the transcoder object associated with @var{port}, or @code{#f}
 if the port has no transcoder."
-  (cond ((port-encoding port)
-         => (lambda (encoding)
-              (make-transcoder
-               encoding
-               (native-eol-style)
-               (case (port-conversion-strategy port)
-                 ((error) 'raise)
-                 ((substitute) 'replace)
-                 (else
-                  (assertion-violation 'port-transcoder
-                                       "unsupported error handling mode"))))))
-        (else
-         #f)))
+  (and (textual-port? port)
+       ;; All textual ports have transcoders.
+       (make-transcoder
+        (port-encoding port)
+        (native-eol-style)
+        (case (port-conversion-strategy port)
+          ((error) 'raise)
+          ((substitute) 'replace)
+          (else
+           (assertion-violation 'port-transcoder
+                                "unsupported error handling mode"))))))
 
 (define (binary-port? port)
-  "Returns @code{#t} if @var{port} does not have an associated encoding,
-@code{#f} otherwise."
-  (not (port-encoding port)))
+  "Return @code{#t} if @var{port} appears to be a binary port, else
+return @code{#f}.  Note that Guile does not currently distinguish
+between binary and textual ports, so this predicate is not a reliable
+indicator of whether the port was created as a binary port.  Currently,
+it returns @code{#t} if and only if the port encoding is ``ISO-8859-1'',
+because Guile uses this encoding when creating a binary port."
+  (equal? (port-encoding port) "ISO-8859-1"))
 
 (define (textual-port? port)
-  "Always returns @code{#t}, as all ports can be used for textual I/O in
-Guile."
+  "Return @code{#t} if @var{port} appears to be a textual port, else
+return @code{#f}.  Note that Guile does not currently distinguish
+between binary and textual ports, so this predicate is not a reliable
+indicator of whether the port was created as a textual port.  Currently,
+it always returns @code{#t}, because all ports can be used for textual
+I/O in Guile."
   #t)
 
 (define (port-eof? port)
@@ -286,15 +331,6 @@ read from/written to in @var{port}."
   (and (false-if-exception (set-port-position! port (port-position port)))
        #t))
 
-(define (call-with-port port proc)
-  "Call @var{proc}, passing it @var{port} and closing @var{port} upon exit of
-@var{proc}.  Return the return values of @var{proc}."
-  (call-with-values
-      (lambda () (proc port))
-    (lambda vals
-      (close-port port)
-      (apply values vals))))
-
 (define* (call-with-bytevector-output-port proc #:optional (transcoder #f))
   (receive (port extract) (open-bytevector-output-port transcoder)
     (call-with-port port proc)
@@ -302,52 +338,70 @@ read from/written to in @var{port}."
 
 (define (open-string-input-port str)
   "Open an input port that will read from @var{str}."
-  (with-fluids ((%default-port-encoding "UTF-8"))
-    (open-input-string str)))
+  (open-input-string str))
+
+(define (r6rs-open filename mode buffer-mode transcoder)
+  (let ((port (with-i/o-filename-conditions filename
+                (lambda ()
+                  (with-fluids ((%default-port-encoding #f))
+                    (open filename mode))))))
+    (setvbuf port buffer-mode)
+    (when transcoder
+      (set-port-encoding! port (transcoder-codec transcoder)))
+    port))
+
+(define (file-options->mode file-options base-mode)
+  (logior base-mode
+          (if (enum-set-member? 'no-create file-options)
+              0
+              O_CREAT)
+          (if (enum-set-member? 'no-truncate file-options)
+              0
+              O_TRUNC)
+          (if (enum-set-member? 'no-fail file-options)
+              0
+              O_EXCL)))
 
 (define* (open-file-input-port filename
                                #:optional
                                (file-options (file-options))
                                (buffer-mode (buffer-mode block))
-                               maybe-transcoder)
-  (let ((port (with-i/o-filename-conditions filename
-                (lambda ()
-                  (with-fluids ((%default-port-encoding #f))
-                    (open filename O_RDONLY))))))
-    (cond (maybe-transcoder
-           (set-port-encoding! port (transcoder-codec maybe-transcoder))))
-    port))
+                               transcoder)
+  "Return an input port for reading from @var{filename}."
+  (r6rs-open filename O_RDONLY buffer-mode transcoder))
+
+(define* (open-file-input/output-port filename
+                                      #:optional
+                                      (file-options (file-options))
+                                      (buffer-mode (buffer-mode block))
+                                      transcoder)
+  "Return a port for reading from and writing to @var{filename}."
+  (r6rs-open filename
+             (file-options->mode file-options O_RDWR)
+             buffer-mode
+             transcoder))
 
 (define (open-string-output-port)
   "Return two values: an output port that will collect characters written to it
 as a string, and a thunk to retrieve the characters associated with that port."
-  (let ((port (with-fluids ((%default-port-encoding "UTF-8"))
-                (open-output-string))))
+  (let ((port (open-output-string)))
     (values port
-            (lambda () (get-output-string port)))))
+            (lambda ()
+              (let ((s (get-output-string port)))
+                (seek port 0 SEEK_SET)
+                (truncate-file port 0)
+                s)))))
 
 (define* (open-file-output-port filename
                                 #:optional
                                 (file-options (file-options))
                                 (buffer-mode (buffer-mode block))
                                 maybe-transcoder)
-  (let* ((flags (logior O_WRONLY
-                        (if (enum-set-member? 'no-create file-options)
-                            0
-                            O_CREAT)
-                        (if (enum-set-member? 'no-truncate file-options)
-                            0
-                            O_TRUNC)
-                        (if (enum-set-member? 'no-fail file-options)
-                            0
-                            O_EXCL)))
-         (port (with-i/o-filename-conditions filename
-                 (lambda ()
-                   (with-fluids ((%default-port-encoding #f))
-                     (open filename flags))))))
-    (cond (maybe-transcoder
-           (set-port-encoding! port (transcoder-codec maybe-transcoder))))
-    port))
+  "Return an output port for writing to @var{filename}."
+  (r6rs-open filename
+             (file-options->mode file-options O_WRONLY)
+             buffer-mode
+             maybe-transcoder))
 
 (define (call-with-string-output-port proc)
   "Call @var{proc}, passing it a string output port. When @var{proc} returns,
@@ -368,6 +422,16 @@ return the characters accumulated in that port."
                           close)
                   "w"))
 
+(define (output-port-buffer-mode port)
+  "Return @code{none} if @var{port} is unbuffered, @code{line} if it is
+line buffered, or @code{block} otherwise."
+  (let ((buffering (bytevector-length
+                    (port-buffer-bytevector (port-write-buffer port)))))
+    (cond
+     ((= buffering 1) 'none)
+     ((port-line-buffered? port) 'line)
+     (else 'block))))
+
 (define (flush-output-port port)
   (force-output port))
 
@@ -382,7 +446,7 @@ return the characters accumulated in that port."
 
 (define-syntax with-i/o-encoding-error
   (syntax-rules ()
-    "Convert Guile throws to `encoding-error' to `&i/o-encoding-error'."
+    "Convert Guile throws to `encoding-error' to `&i/o-encoding'."
     ((_ body ...)
      ;; XXX: This is heavyweight for small functions like `put-char'.
      (with-throw-handler 'encoding-error
@@ -423,7 +487,7 @@ return the characters accumulated in that port."
 
 (define-syntax with-i/o-decoding-error
   (syntax-rules ()
-    "Convert Guile throws to `decoding-error' to `&i/o-decoding-error'."
+    "Convert Guile throws to `decoding-error' to `&i/o-decoding'."
     ((_ body ...)
      ;; XXX: This is heavyweight for small functions like `get-char' and
      ;; `lookahead-char'.
@@ -443,7 +507,7 @@ return the characters accumulated in that port."
   (with-textual-input-conditions port (read-line port 'trim)))
 
 (define (get-string-all port)
-  (with-textual-input-conditions port (read-delimited "" port 'concat)))
+  (with-textual-input-conditions port (read-string port)))
 
 (define (get-string-n port count)
   "Read up to @var{count} characters from @var{port}.
