@@ -1,6 +1,6 @@
 ;;; Repl commands
 
-;; Copyright (C) 2001, 2009, 2010, 2011, 2012, 2013 Free Software Foundation, Inc.
+;; Copyright (C) 2001, 2009, 2010, 2011, 2012, 2013, 2020, 2021 Free Software Foundation, Inc.
 
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -22,15 +22,18 @@
 (define-module (system repl command)
   #:use-module (system base syntax)
   #:use-module (system base pmatch)
-  #:use-module (system base compile)
+  #:autoload   (system base compile) (compile-file)
   #:use-module (system repl common)
   #:use-module (system repl debug)
-  #:use-module (system vm disassembler)
+  #:autoload   (system vm disassembler) (disassemble-image
+                                         disassemble-program
+                                         disassemble-file)
   #:use-module (system vm loader)
   #:use-module (system vm program)
   #:use-module (system vm trap-state)
   #:use-module (system vm vm)
-  #:autoload (system base language) (lookup-language language-reader)
+  #:autoload (system base language) (lookup-language language-reader
+                                     language-title language-name)
   #:autoload (system vm trace) (call-with-trace)
   #:use-module (ice-9 format)
   #:use-module (ice-9 session)
@@ -41,7 +44,7 @@
   #:use-module ((ice-9 pretty-print) #:select ((pretty-print . pp)))
   #:use-module ((system vm inspect) #:select ((inspect . %inspect)))
   #:use-module (rnrs bytevectors)
-  #:use-module (statprof)
+  #:autoload   (statprof) (statprof)
   #:export (meta-command define-meta-command))
 
 
@@ -54,11 +57,11 @@
     (module   (module m) (import use) (load l) (reload re) (binding b) (in))
     (language (language L))
     (compile  (compile c) (compile-file cc)
-              (expand exp) (optimize opt)
+              (expand exp) (optimize opt) (optimize-cps optx)
 	      (disassemble x) (disassemble-file xx))
     (profile  (time t) (profile pr) (trace tr))
     (debug    (backtrace bt) (up) (down) (frame fr)
-              (procedure proc) (locals) (error-message error)
+              (locals) (error-message error)
               (break br bp) (break-at-source break-at bs)
               (step s) (step-instruction si)
               (next n) (next-instruction ni)
@@ -487,6 +490,11 @@ Run the optimizer on a piece of code and print the result."
     (run-hook before-print-hook x)
     (pp x)))
 
+(define-meta-command (optimize-cps repl (form))
+  "optimize-cps EXP
+Run the CPS optimizer on a piece of code and print the result."
+  (repl-optimize-cps repl (repl-parse repl form)))
+
 (define-meta-command (disassemble repl (form))
   "disassemble EXP
 Disassemble a compiled procedure."
@@ -645,11 +653,6 @@ With an argument, select a frame by index, then show it."
       (format #t "No such frame.~%"))))
    (else (print-frame cur #:index index))))
 
-(define-stack-command (procedure repl)
-  "procedure
-Print the procedure for the selected frame."
-  (repl-print repl (frame-procedure cur)))
-
 (define-stack-command (locals repl #:key (width (terminal-width)))
   "locals
 Show local variables.
