@@ -169,7 +169,7 @@
       (when (< n (vector-length defs))
         (match (vector-ref defs n)
           (#(_ _ slot _)
-           (bitvector-set! (vector-ref by-slot slot) n #t)
+           (bitvector-set-bit! (vector-ref by-slot slot) n)
            (lp (1+ n))))))
     by-slot))
 
@@ -180,7 +180,8 @@
                  (compute-frame-sizes code parsed initial-frame-size))
                 ((killv) (make-vector (vector-length parsed) #f)))
     (define (kill-slot! n slot)
-      (bit-set*! (vector-ref killv n) (vector-ref defs-by-slot slot) #t))
+      (bitvector-set-bits! (vector-ref killv n)
+                           (vector-ref defs-by-slot slot)))
     (let lp ((n 0))
       (when (< n (vector-length killv))
         (vector-set! killv n (make-bitvector (vector-length defs) #f))
@@ -223,12 +224,12 @@
          (outv (make-vector len #f))
          (tmp (make-bitvector (vector-length defs) #f)))
     (define (bitvector-copy! dst src)
-      (bitvector-fill! dst #f)
-      (bit-set*! dst src #t))
+      (bitvector-clear-all-bits! dst)
+      (bitvector-set-bits! dst src))
     (define (bitvector-meet! accum src)
       (bitvector-copy! tmp src)
-      (bit-invert! tmp)
-      (bit-set*! accum tmp #f))
+      (bitvector-flip-all-bits! tmp)
+      (bitvector-clear-bits! accum tmp))
 
     (let lp ((n 0))
       (when (< n len)
@@ -243,8 +244,10 @@
               (out (vector-ref outv n))
               (kill (vector-ref killv n))
               (gen (vector-ref genv n)))
-          (let ((out-count (or changed? (bit-count #t out))))
-            (bitvector-fill! in (not (zero? n)))
+          (let ((out-count (or changed? (bitvector-count out))))
+            (if (zero? n)
+                (bitvector-clear-all-bits! in)
+                (bitvector-set-all-bits! in))
             (let lp ((preds (vector-ref preds n)))
               (match preds
                 (() #t)
@@ -253,12 +256,12 @@
                    (bitvector-meet! in (vector-ref outv pred)))
                  (lp preds))))
             (bitvector-copy! out in)
-            (bit-set*! out kill #f)
+            (bitvector-clear-bits! out kill)
             (for-each (lambda (def)
-                        (bitvector-set! out def #t))
+                        (bitvector-set-bit! out def))
                       gen)
             (lp (1+ n) first?
-                (or changed? (not (eqv? out-count (bit-count #t out))))))))
+                (or changed? (not (eqv? out-count (bitvector-count out))))))))
        ((or changed? first?)
         (lp 0 #f #f))))
 
@@ -280,10 +283,10 @@
                           ;; values defined by the call.
                           (begin
                             (bitvector-copy! tmp (vector-ref inv (1- n)))
-                            (bit-set*! tmp (vector-ref killv (1- n)) #f)
+                            (bitvector-clear-bits! tmp (vector-ref killv (1- n)))
                             tmp))))
             (let lp ((n 0))
-              (let ((n (bit-position #t live n)))
+              (let ((n (bitvector-position live #t n)))
                 (if n
                     (match (vector-ref defs n)
                       (#(name def-offset slot representation)
@@ -378,8 +381,16 @@
         (frame-local-ref frame i 'scm))
        ((find-slot i bindings)
         => (lambda (binding)
-             (let ((val (frame-local-ref frame (binding-slot binding)
-                                         (binding-representation binding))))
+             (let* ((slot (binding-slot binding))
+                    ;; HACK: Avoid out-of-range from frame-local-ref.
+                    ;; Some frames have bindings beyond nlocals.  That
+                    ;; is probably a bug somewhere else, but at least
+                    ;; this workaround allows them to be printed.
+                    (val (if (< slot nlocals)
+                             (frame-local-ref frame slot
+                                              (binding-representation binding))
+                             ;; else #<unspecified>
+                             )))
                ;; It could be that there's a value that isn't clobbered
                ;; by a call but that isn't live after a call either.  In
                ;; that case, if GC runs during the call, the value will
@@ -428,7 +439,7 @@
                  (arity-nopt arity)
                  (arity-keyword-args arity)
                  (arity-has-rest? arity)
-                 1))))
+                 (if (arity-has-closure? arity) 1 0)))))
       ((and (primitive-code? ip)
             (program-arguments-alist (frame-local-ref frame 0 'scm) ip))
        => (lambda (args)
